/* ltt-control.c
 *
 * LTT control module over a netlink socket.
 *
 * Copyright 2005 -
 * Mathieu Desnoyers <mathieu.desnoyers@polymtl.ca>
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/ltt-core.h>
#include <linux/netlink.h>
#include <linux/inet.h>
#include <linux/ip.h>
#include "ltt-control.h"

static struct sock *socket;

void ltt_control_input(struct sock *sk, int len)
{
	struct sk_buff *skb;
	struct nlmsghdr *nlh = NULL;
	u8 *payload = NULL;
	lttctl_peer_msg_t *msg;

	printk(KERN_ALERT "ltt-control ltt_control_input\n");

	while ((skb = skb_dequeue(&sk->sk_receive_queue)) 
					!= NULL) {
		/* process netlink message pointed by skb->data */
		nlh = (struct nlmsghdr *)skb->data;
		payload = NLMSG_DATA(nlh);
		/* process netlink message with header pointed by 
		 * nlh and payload pointed by payload
		 */
		if(nlh->nlmsg_len != sizeof(lttctl_peer_msg_t) + sizeof(struct nlmsghdr)) {
			printk(KERN_ALERT "ltt-control bad message length\n");
			return;
		}
		msg = (lttctl_peer_msg_t*)payload;

		switch(msg->op) {
			case OP_CREATE:
				switch(msg->args.mode) {
					case LTT_TRACE_NORMAL:
						ltt_control(LTT_CONTROL_CREATE_NORMAL_TRACE, msg->trace_name);
						break;
					case LTT_TRACE_FLIGHT:
						ltt_control(LTT_CONTROL_CREATE_FLIGHT_TRACE, msg->trace_name);
						break;
				}
				break;
			case OP_DESTROY:
				ltt_control(LTT_CONTROL_DESTROY_TRACE, msg->trace_name);
				break;
			case OP_START:
				ltt_control(LTT_CONTROL_START, msg->trace_name);
				break;
			case OP_STOP:
				ltt_control(LTT_CONTROL_STOP, msg->trace_name);
				break;
			default:
				printk(KERN_INFO "ltt-control invalid operation\n");
		}
		
	}
}


static int ltt_control_init(void)
{
	printk(KERN_ALERT "ltt-control init\n");

	socket = netlink_kernel_create(NETLINK_LTT,
			ltt_control_input);
	if(socket == NULL) return -EPERM;
	else return 0;
}

static void ltt_control_exit(void)
{
	printk(KERN_ALERT "ltt-control exit\n");

	sock_release(socket->sk_socket);
}


module_init(ltt_control_init)
module_exit(ltt_control_exit)

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Mathieu Desnoyers");
MODULE_DESCRIPTION("Linux Trace Toolkit Controller");

